@@ -4,6 +4,7 @@ from fal import apps  # noqa: F401
 from fal.api import FalServerlessHost, LocalHost, cached, function
 from fal.api import function as isolated  # noqa: F401
 from fal.app import App, endpoint, realtime, wrap_app  # noqa: F401
+from fal.container import ContainerImage
 from fal.sdk import FalServerlessKeyCredentials
 from fal.sync import sync_dir
 
@@ -26,4 +27,5 @@ __all__ = [
     "sync_dir",
     "__version__",
     "version_tuple",
+    "ContainerImage",
 ]
