@@ -2,8 +2,10 @@ from __future__ import annotations
 
 from ._base import (
     AppException,  # noqa: F401
+    AppFileUploadException,  # noqa: F401
     FalServerlessException,  # noqa: F401
     FieldException,  # noqa: F401
+    FileTooLargeError,  # noqa: F401
     RequestCancelledException,  # noqa: F401
 )
 from ._cuda import CUDAOutOfMemoryException  # noqa: F401
