@@ -56,3 +56,18 @@ class RequestCancelledException(FalServerlessException):
     """Exception raised when the request is cancelled by the client."""
 
     message: str = "Request cancelled by the client."
+
+
+@dataclass
+class FileTooLargeError(FalServerlessException):
+    """Exception raised when the file is too large."""
+
+    message: str = "File is too large."
+
+
+@dataclass
+class AppFileUploadException(FalServerlessException):
+    """Raised when file upload fails"""
+
+    message: str
+    relative_path: str
