@@ -7,5 +7,5 @@ class UnauthenticatedException(FalServerlessException):
     def __init__(self) -> None:
         super().__init__(
             "You must be authenticated. "
-            "Login via `fal auth login` or make sure to setup fal keys correctly."
+            "Use [bold]fal auth login[/] or [bold]fal profile key[/] to set your fal key."  # noqa: E501
         )
