@@ -38,6 +38,7 @@ FAL_SERVERLESS_DEFAULT_KEEP_ALIVE = 10
 FAL_SERVERLESS_DEFAULT_MAX_MULTIPLEXING = 1
 FAL_SERVERLESS_DEFAULT_MIN_CONCURRENCY = 0
 FAL_SERVERLESS_DEFAULT_CONCURRENCY_BUFFER = 0
+FAL_SERVERLESS_DEFAULT_CONCURRENCY_BUFFER_PERC = 0
 ALIAS_AUTH_MODES = ["public", "private", "shared"]
 
 logger = get_logger(__name__)
@@ -45,7 +46,8 @@ logger = get_logger(__name__)
 patch_pickle()
 
 
-AuthMode = Optional[Literal["public", "private", "shared"]]
+AuthModeLiteral = Literal["public", "private", "shared"]
+DeploymentStrategyLiteral = Literal["recreate", "rolling"]
 
 
 class ServerCredentials:
@@ -228,6 +230,12 @@ class HostedRunStatus:
     state: HostedRunState
 
 
+@dataclass
+class File:
+    hash: str
+    relative_path: str
+
+
 @dataclass
 class ApplicationInfo:
     application_id: str
@@ -237,6 +245,7 @@ class ApplicationInfo:
     active_runners: int
     min_concurrency: int
     concurrency_buffer: int
+    concurrency_buffer_perc: int
     machine_types: list[str]
     request_timeout: int
     startup_timeout: int
@@ -255,6 +264,7 @@ class AliasInfo:
     active_runners: int
     min_concurrency: int
     concurrency_buffer: int
+    concurrency_buffer_perc: int
     machine_types: list[str]
     request_timeout: int
     startup_timeout: int
@@ -265,6 +275,7 @@ class RunnerState(Enum):
     RUNNING = "running"
     PENDING = "pending"
     SETUP = "setup"
+    DOCKER_PULL = "docker_pull"
     DEAD = "dead"
     UNKNOWN = "unknown"
 
@@ -278,6 +289,8 @@ class RunnerState(Enum):
             return RunnerState.SETUP
         elif proto is isolate_proto.RunnerInfo.State.DEAD:
             return RunnerState.DEAD
+        elif proto is isolate_proto.RunnerInfo.State.DOCKER_PULL:
+            return RunnerState.DOCKER_PULL
         else:
             return RunnerState.UNKNOWN
 
@@ -400,6 +413,7 @@ def _from_grpc_application_info(
         active_runners=message.active_runners,
         min_concurrency=message.min_concurrency,
         concurrency_buffer=message.concurrency_buffer,
+        concurrency_buffer_perc=message.concurrency_buffer_perc,
         machine_types=list(message.machine_types),
         request_timeout=message.request_timeout,
         startup_timeout=message.startup_timeout,
@@ -429,6 +443,7 @@ def _from_grpc_alias_info(message: isolate_proto.AliasInfo) -> AliasInfo:
         active_runners=message.active_runners,
         min_concurrency=message.min_concurrency,
         concurrency_buffer=message.concurrency_buffer,
+        concurrency_buffer_perc=message.concurrency_buffer_perc,
         machine_types=list(message.machine_types),
         request_timeout=message.request_timeout,
         startup_timeout=message.startup_timeout,
@@ -521,6 +536,7 @@ class MachineRequirements:
     max_multiplexing: int | None = None
     min_concurrency: int | None = None
     concurrency_buffer: int | None = None
+    concurrency_buffer_perc: int | None = None
     request_timeout: int | None = None
     startup_timeout: int | None = None
 
@@ -614,14 +630,16 @@ class FalServerlessConnection:
         function: Callable[..., ResultT],
         environments: list[isolate_proto.EnvironmentDefinition],
         application_name: str | None = None,
-        auth_mode: AuthMode = None,
+        auth_mode: Optional[AuthModeLiteral] = None,
         *,
+        source_code: str | None = None,
         serialization_method: str = _DEFAULT_SERIALIZATION_METHOD,
         machine_requirements: MachineRequirements | None = None,
         metadata: dict[str, Any] | None = None,
-        deployment_strategy: Literal["recreate", "rolling"] = "recreate",
+        deployment_strategy: DeploymentStrategyLiteral,
         scale: bool = True,
         private_logs: bool = False,
+        files: list[File] | None = None,
     ) -> Iterator[isolate_proto.RegisterApplicationResult]:
         wrapped_function = to_serialized_object(function, serialization_method)
         if machine_requirements:
@@ -640,6 +658,7 @@ class FalServerlessConnection:
                 max_concurrency=machine_requirements.max_concurrency,
                 min_concurrency=machine_requirements.min_concurrency,
                 concurrency_buffer=machine_requirements.concurrency_buffer,
+                concurrency_buffer_perc=machine_requirements.concurrency_buffer_perc,
                 max_multiplexing=machine_requirements.max_multiplexing,
                 request_timeout=machine_requirements.request_timeout,
                 startup_timeout=machine_requirements.startup_timeout,
@@ -647,6 +666,12 @@ class FalServerlessConnection:
         else:
             wrapped_requirements = None
 
+        if files:
+            files = [
+                isolate_proto.File(hash=file.hash, relative_path=file.relative_path)
+                for file in files
+            ]
+
         if auth_mode == "public":
             auth = isolate_proto.ApplicationAuthMode.PUBLIC
         elif auth_mode == "shared":
@@ -675,6 +700,8 @@ class FalServerlessConnection:
             deployment_strategy=deployment_strategy_proto,
             scale=scale,
             private_logs=private_logs,
+            files=files,
+            source_code=source_code,
         )
         for partial_result in self.stub.RegisterApplication(request):
             yield from_grpc(partial_result)
@@ -690,6 +717,7 @@ class FalServerlessConnection:
         max_concurrency: int | None = None,
         min_concurrency: int | None = None,
         concurrency_buffer: int | None = None,
+        concurrency_buffer_perc: int | None = None,
         request_timeout: int | None = None,
         startup_timeout: int | None = None,
         valid_regions: list[str] | None = None,
@@ -702,6 +730,7 @@ class FalServerlessConnection:
             max_concurrency=max_concurrency,
             min_concurrency=min_concurrency,
             concurrency_buffer=concurrency_buffer,
+            concurrency_buffer_perc=concurrency_buffer_perc,
             request_timeout=request_timeout,
             startup_timeout=startup_timeout,
             valid_regions=valid_regions,
@@ -736,6 +765,7 @@ class FalServerlessConnection:
         serialization_method: str = _DEFAULT_SERIALIZATION_METHOD,
         machine_requirements: MachineRequirements | None = None,
         setup_function: Callable[[], InputT] | None = None,
+        files: list[File] | None = None,
     ) -> Iterator[HostedRunResult[ResultT]]:
         wrapped_function = to_serialized_object(function, serialization_method)
         if machine_requirements:
@@ -755,16 +785,22 @@ class FalServerlessConnection:
                 max_multiplexing=machine_requirements.max_multiplexing,
                 min_concurrency=machine_requirements.min_concurrency,
                 concurrency_buffer=machine_requirements.concurrency_buffer,
+                concurrency_buffer_perc=machine_requirements.concurrency_buffer_perc,
                 request_timeout=machine_requirements.request_timeout,
                 startup_timeout=machine_requirements.startup_timeout,
             )
         else:
             wrapped_requirements = None
-
+        if files:
+            files = [
+                isolate_proto.File(hash=file.hash, relative_path=file.relative_path)
+                for file in files
+            ]
         request = isolate_proto.HostedRun(
             function=wrapped_function,
             environments=environments,
             machine_requirements=wrapped_requirements,
+            files=files,
         )
         if setup_function:
             request.setup_func.MergeFrom(
@@ -780,7 +816,7 @@ class FalServerlessConnection:
         self,
         alias: str,
         revision: str,
-        auth_mode: AuthMode,
+        auth_mode: Optional[AuthModeLiteral],
     ) -> AliasInfo:
         if auth_mode == "public":
             auth = isolate_proto.ApplicationAuthMode.PUBLIC
