@@ -1,12 +1,15 @@
 from __future__ import annotations
 
+import errno
 import hashlib
+import os
 import shutil
 import subprocess
 import sys
-from functools import lru_cache
+import time
+from contextlib import suppress
 from pathlib import Path, PurePath
-from tempfile import TemporaryDirectory
+from tempfile import NamedTemporaryFile, TemporaryDirectory
 from urllib.parse import urlparse
 from urllib.request import Request, urlopen
 
@@ -40,8 +43,10 @@ def _hash_url(url: str) -> str:
     return hashlib.sha256(url.encode("utf-8")).hexdigest()
 
 
-@lru_cache
-def _get_remote_file_properties(url: str) -> tuple[str, int]:
+def _get_remote_file_properties(
+    url: str,
+    request_headers: dict[str, str] | None = None,
+) -> tuple[str, int]:
     """Retrieves the file name and content length of a remote file.
 
     This function sends an HTTP request to the remote URL and retrieves the
@@ -60,11 +65,15 @@ def _get_remote_file_properties(url: str) -> tuple[str, int]:
 
     Args:
         url: The URL of the remote file.
+        request_headers: A dictionary containing additional headers to be included in
+            the HTTP request.
 
     Returns:
         A tuple containing the file name and the content length of the remote file.
     """
-    request = Request(url, headers=TEMP_HEADERS)
+    headers = {**TEMP_HEADERS, **(request_headers or {})}
+    request = Request(url, headers=headers)
+
     with urlopen(request) as response:
         file_name = response.headers.get_filename()
         content_length = int(response.headers.get("Content-Length", -1))
@@ -78,10 +87,15 @@ def _get_remote_file_properties(url: str) -> tuple[str, int]:
             url_path = parsed_url.path
             file_name = Path(url_path).name or _hash_url(url)
 
+    # file name can still contain a forward slash if the server returns a relative path
+    file_name = Path(file_name).name
+
     return file_name, content_length
 
 
-def _file_content_length_matches(url: str, file_path: Path) -> bool:
+def _file_content_length_matches(
+    url: str, file_path: Path, request_headers: dict[str, str] | None = None
+) -> bool:
     """Check if the local file's content length matches the expected remote
     file's content length.
 
@@ -95,13 +109,15 @@ def _file_content_length_matches(url: str, file_path: Path) -> bool:
     Args:
         url: The URL of the remote file.
         file_path: The local path to the file being compared.
+        request_headers: A dictionary containing additional headers to be included in
+            the HTTP request.
 
     Returns:
         bool: `True` if the local file's content length matches the remote file's
             content length, `False` otherwise.
     """
     local_file_content_length = file_path.stat().st_size
-    remote_file_content_length = _get_remote_file_properties(url)[1]
+    remote_file_content_length = _get_remote_file_properties(url, request_headers)[1]
 
     return local_file_content_length == remote_file_content_length
 
@@ -111,11 +127,13 @@ def download_file(
     target_dir: str | Path,
     *,
     force: bool = False,
+    request_headers: dict[str, str] | None = None,
+    filesize_limit: int | None = None,
 ) -> Path:
     """Downloads a file from the specified URL to the target directory.
 
     The function downloads the file from the given URL and saves it in the specified
-    target directory.
+    target directory, provided it is below the given filesize limit.
 
     It also checks whether the local file already exists and whether its content length
     matches the expected content length from the remote file. If the local file already
@@ -134,6 +152,11 @@ def download_file(
         force: If `True`, the file is downloaded even if it already exists locally and
             its content length matches the expected content length from the remote file.
             Defaults to `False`.
+        request_headers: A dictionary containing additional headers to be included in
+            the HTTP request. Defaults to `None`.
+        filesize_limit: An integer specifying the maximum downloadable size,
+            in megabytes. Defaults to `None`.
+
 
     Returns:
         A Path object representing the full path to the downloaded file.
@@ -142,7 +165,22 @@ def download_file(
         ValueError: If the provided `file_name` contains a forward slash ('/').
         DownloadError: If an error occurs during the download process.
     """
-    file_name = _get_remote_file_properties(url)[0]
+    ONE_MB = 1024**2
+
+    try:
+        file_name, expected_filesize = _get_remote_file_properties(url, request_headers)
+    except Exception as e:
+        print(f"Got error: {e}")
+        raise DownloadError(f"Failed to get remote file properties for {url}") from e
+
+    expected_filesize_mb = expected_filesize / ONE_MB
+
+    if filesize_limit is not None and expected_filesize_mb > filesize_limit:
+        raise DownloadError(
+            f"""File to be downloaded is of size {expected_filesize_mb},
+                which is over the limit of {filesize_limit}"""
+        )
+
     if "/" in file_name:
         raise ValueError(f"File name '{file_name}' cannot contain a slash.")
 
@@ -156,7 +194,7 @@ def download_file(
 
     if (
         target_path.exists()
-        and _file_content_length_matches(url, target_path)
+        and _file_content_length_matches(url, target_path, request_headers)
         and not force
     ):
         return target_path
@@ -170,7 +208,12 @@ def download_file(
     target_path.parent.mkdir(parents=True, exist_ok=True)
 
     try:
-        _download_file_python(url=url, target_path=target_path)
+        _download_file_python(
+            url=url,
+            target_path=target_path,
+            request_headers=request_headers,
+            filesize_limit=filesize_limit,
+        )
     except Exception as e:
         msg = f"Failed to download {url} to {target_path}"
 
@@ -181,45 +224,71 @@ def download_file(
     return target_path
 
 
-def _download_file_python(url: str, target_path: Path | str) -> Path:
+def _download_file_python(
+    url: str,
+    target_path: Path | str,
+    request_headers: dict[str, str] | None = None,
+    filesize_limit: int | None = None,
+) -> Path:
     """Download a file from a given URL and save it to a specified path using a
     Python interface.
 
     Args:
         url: The URL of the file to be downloaded.
         target_path: The path where the downloaded file will be saved.
+        request_headers: A dictionary containing additional headers to be included in
+            the HTTP request. Defaults to `None`.
+        filesize_limit: A integer value specifying how many megabytes can be
+            downloaded at maximum. Defaults to `None`.
 
     Returns:
         The path where the downloaded file has been saved.
     """
-    import shutil
-    import tempfile
-
-    with tempfile.NamedTemporaryFile(delete=False) as temp_file:
+    basename = os.path.basename(target_path)
+    # NOTE: using the same directory to avoid potential copies across temp fs and target
+    # fs, and also to be able to atomically rename a downloaded file into place.
+    with NamedTemporaryFile(
+        delete=False,
+        dir=os.path.dirname(target_path),
+        prefix=f"{basename}.tmp",
+    ) as temp_file:
         try:
             file_path = temp_file.name
 
-            for (progress, total_size) in _stream_url_data_to_file(url, temp_file.name):
+            for progress, total_size in _stream_url_data_to_file(
+                url,
+                temp_file.name,
+                request_headers=request_headers,
+                filesize_limit=filesize_limit,
+            ):
                 if total_size:
                     progress_msg = f"Downloading {url} ... {progress:.2%}"
                 else:
                     progress_msg = f"Downloading {url} ... {progress:.2f} MB"
+
                 print(progress_msg, end="\r\n")
 
-            # Move the file when the file is downloaded completely. Since the
-            # file used is temporary, in a case of an interruption, the downloaded
-            # content will be lost. So, it is safe to redownload the file in such cases.
-            shutil.move(file_path, target_path)
+            # NOTE: Atomically renaming the file into place when the file is downloaded
+            # completely.
+            #
+            # Since the file used is temporary, in a case of an interruption, the
+            # downloaded content will be lost. So, it is safe to redownload the file in
+            # such cases.
+            os.rename(file_path, target_path)
 
-        except Exception as error:
-            raise error
         finally:
             Path(temp_file.name).unlink(missing_ok=True)
 
     return Path(target_path)
 
 
-def _stream_url_data_to_file(url: str, file_path: str, chunk_size_in_mb: int = 64):
+def _stream_url_data_to_file(
+    url: str,
+    file_path: str,
+    chunk_size_in_mb: int = 64,
+    request_headers: dict[str, str] | None = None,
+    filesize_limit: int | None = None,
+):
     """Download data from a URL and stream it to a file.
 
     Note:
@@ -233,6 +302,10 @@ def _stream_url_data_to_file(url: str, file_path: str, chunk_size_in_mb: int = 6
         file_path: The path to the file where the downloaded data will be saved.
         chunk_size_in_mb: The size of each download chunk in megabytes.
             Defaults to 64.
+        request_headers: A dictionary containing additional headers to be included in
+            the HTTP request. Defaults to `None`.
+        filesize_limit: An integer specifying how many megabytes can be
+            downloaded at maximum. Defaults to `None`.
 
     Yields:
         A tuple containing two elements:
@@ -244,7 +317,8 @@ def _stream_url_data_to_file(url: str, file_path: str, chunk_size_in_mb: int = 6
     """
     ONE_MB = 1024**2
 
-    request = Request(url, headers=TEMP_HEADERS)
+    headers = {**TEMP_HEADERS, **(request_headers or {})}
+    request = Request(url, headers=headers)
 
     received_size = 0
     total_size = 0
@@ -255,6 +329,11 @@ def _stream_url_data_to_file(url: str, file_path: str, chunk_size_in_mb: int = 6
             f_stream.write(data)
 
             received_size = f_stream.tell()
+            if filesize_limit is not None and received_size > filesize_limit:
+                raise DownloadError(
+                    f"""Attempted to download more data {received_size}
+                        than the set limit of {filesize_limit}"""
+                )
 
             if total_size:
                 progress = received_size / total_size
@@ -267,7 +346,17 @@ def _stream_url_data_to_file(url: str, file_path: str, chunk_size_in_mb: int = 6
         raise DownloadError("Received less data than expected from the server.")
 
 
-def download_model_weights(url: str, force: bool = False):
+def _mark_used_dir(dir: Path):
+    used_file = dir / ".fal_used"
+    day_ago = time.time() - 86400
+    if not used_file.exists() or used_file.stat().st_mtime < day_ago:
+        # Touch a last-used file to indicate that the weights have been used
+        used_file.touch()
+
+
+def download_model_weights(
+    url: str, force: bool = False, request_headers: dict[str, str] | None = None
+) -> Path:
     """Downloads model weights from the specified URL and saves them to a
     predefined directory.
 
@@ -284,6 +373,8 @@ def download_model_weights(url: str, force: bool = False):
         force: If `True`, the model weights are downloaded even if they already exist
             locally and their content length matches the expected content length from
             the remote file. Defaults to `False`.
+        request_headers: A dictionary containing additional headers to be included in
+            the HTTP request. Defaults to `None`.
 
     Returns:
         A Path object representing the full path to the downloaded model weights.
@@ -293,18 +384,34 @@ def download_model_weights(url: str, force: bool = False):
 
     if weights_dir.exists() and not force:
         try:
-            weights_path = next(weights_dir.glob("*"))
+            # TODO: sometimes the directory can hold multiple files
+            # Example:
+            # .fal/model_weights/00155dc2d9579360d577d1a87d31b52c21135c14a5f44fcbab36fbb8352f3e0d  # noqa: E501
+            # We need to either not allow multiple files in the directory or
+            # find the one that is the most recently used.
+            weights_path = next(
+                # Ignore .fal dotfiles since they are metadata files
+                f
+                for f in weights_dir.glob("*")
+                if not f.name.startswith(".fal")
+            )
+            _mark_used_dir(weights_dir)
             return weights_path
         # The model weights directory is empty, so we need to download the weights
         except StopIteration:
             pass
 
-    return download_file(
+    path = download_file(
         url,
         target_dir=weights_dir,
         force=force,
+        request_headers=request_headers,
     )
 
+    _mark_used_dir(weights_dir)
+
+    return path
+
 
 def clone_repository(
     https_url: str,
@@ -340,13 +447,26 @@ def clone_repository(
         A Path object representing the full path to the cloned Git repository.
     """
     target_dir = target_dir or FAL_REPOSITORY_DIR  # type: ignore[assignment]
-    repo_name = repo_name or Path(https_url).stem
+
+    if repo_name is None:
+        repo_name = Path(https_url).stem
+        if commit_hash:
+            if len(commit_hash) < 8:
+                raise ValueError(f"Commit hash '{commit_hash}' is too short.")
+            repo_name += f"-{commit_hash[:8]}"
 
     local_repo_path = Path(target_dir) / repo_name  # type: ignore[arg-type]
 
     if local_repo_path.exists():
-        local_repo_commit_hash = _get_git_revision_hash(local_repo_path)
-        if local_repo_commit_hash == commit_hash and not force:
+        local_repo_commit_hash = _git_rev_parse(local_repo_path, "HEAD")
+        full_commit_hash = (
+            _git_rev_parse(local_repo_path, commit_hash) if commit_hash else None
+        )
+        if (
+            full_commit_hash
+            and local_repo_commit_hash == full_commit_hash
+            and not force
+        ):
             if include_to_path:
                 __add_local_path_to_sys_path(local_repo_path)
             return local_repo_path
@@ -362,37 +482,59 @@ def clone_repository(
                     f"Forcing re-download."
                 )
             print(f"Removing the existing repository: {local_repo_path} ")
-            shutil.rmtree(local_repo_path)
+            with TemporaryDirectory(
+                dir=target_dir, suffix=f"{local_repo_path.name}.tmp.old"
+            ) as tmp_dir:
+                with suppress(FileNotFoundError):
+                    # repository might be already deleted by another worker
+                    os.rename(local_repo_path, tmp_dir)
+                    # sometimes seeing FileNotFoundError even here on juicefs
+                    shutil.rmtree(tmp_dir, ignore_errors=True)
 
-    # Temporary directory to download the repo into.
-    temp_dir = TemporaryDirectory()
-    temp_dir_path = temp_dir.name
+    # NOTE: using the target_dir to be able to avoid potential copies across temp fs
+    # and target fs, and also to be able to atomically rename repo_name dir into place
+    # when we are done setting it up.
+    os.makedirs(target_dir, exist_ok=True)  # type: ignore[arg-type]
+    with TemporaryDirectory(
+        dir=target_dir,
+        suffix=f"{local_repo_path.name}.tmp",
+    ) as temp_dir:
+        try:
+            print(f"Cloning the repository '{https_url}' .")
 
-    try:
-        print(f"Cloning the repository '{https_url}' .")
+            # Clone with disabling the logs and advices for detached HEAD state.
+            clone_command = [
+                "git",
+                "clone",
+                "--recursive",
+                https_url,
+                temp_dir,
+            ]
+            subprocess.check_call(clone_command)
 
-        # Clone with disabling the logs and advices for detached HEAD state.
-        clone_command = [
-            "git",
-            "clone",
-            "--recursive",
-            https_url,
-            temp_dir_path,
-        ]
-        subprocess.check_call(clone_command)
+            if commit_hash:
+                checkout_command = ["git", "checkout", commit_hash]
+                subprocess.check_call(checkout_command, cwd=temp_dir)
+                subprocess.check_call(
+                    ["git", "submodule", "update", "--init", "--recursive"],
+                    cwd=temp_dir,
+                )
 
-        if commit_hash:
-            checkout_command = ["git", "checkout", commit_hash]
-            subprocess.check_call(checkout_command, cwd=temp_dir_path)
+            # NOTE: Atomically renaming the repository directory into place when the
+            # clone and checkout are done.
+            try:
+                os.rename(temp_dir, local_repo_path)
+            except OSError as error:
+                shutil.rmtree(temp_dir, ignore_errors=True)
 
-        # Move the repository when the clone and checkout is finished.
-        shutil.move(temp_dir_path, local_repo_path)
+                # someone beat us to it, assume it's good
+                if error.errno != errno.ENOTEMPTY:
+                    raise
+                print(f"{local_repo_path} already exists, skipping rename")
 
-    except Exception as error:
-        print(f"{error}\nFailed to clone repository '{https_url}' .")
-        temp_dir.cleanup()
-
-        raise error
+        except Exception as error:
+            print(f"{error}\nFailed to clone repository '{https_url}' .")
+            raise error
 
     if include_to_path:
         __add_local_path_to_sys_path(local_repo_path)
@@ -407,12 +549,12 @@ def __add_local_path_to_sys_path(local_path: Path | str):
         sys.path.insert(0, local_path_str)
 
 
-def _get_git_revision_hash(repo_path: Path) -> str:
+def _git_rev_parse(repo_path: Path, ref: str) -> str:
     import subprocess
 
     try:
         return subprocess.check_output(
-            ["git", "rev-parse", "HEAD"],
+            ["git", "rev-parse", ref],
             cwd=repo_path,
             text=True,
             stderr=subprocess.STDOUT,
