@@ -14,6 +14,7 @@ class LoadedFunction:
     endpoints: list[str]
     app_name: str | None
     app_auth: str | None
+    source_code: str | None
 
 
 def load_function_from(
@@ -31,9 +32,7 @@ def load_function_from(
         fal_objects = {
             obj_name: obj
             for obj_name, obj in module.items()
-            if isinstance(obj, type)
-            and issubclass(obj, fal.App)
-            and hasattr(obj, "app_name")
+            if isinstance(obj, type) and issubclass(obj, fal.App) and obj is not fal.App
         }
         if len(fal_objects) == 0:
             raise FalServerlessError("No fal.App found in the module.")
@@ -58,6 +57,10 @@ def load_function_from(
     fal._serialization.include_package_from_path(file_path)
 
     target = module[function_name]
+
+    with open(file_path) as f:
+        source_code = f.read()
+
     endpoints = ["/"]
     if isinstance(target, type) and issubclass(target, App):
         app_name = target.app_name
@@ -69,4 +72,6 @@ def load_function_from(
         raise FalServerlessError(
             f"Function '{function_name}' is not a fal.function or a fal.App"
         )
-    return LoadedFunction(target, endpoints, app_name=app_name, app_auth=app_auth)
+    return LoadedFunction(
+        target, endpoints, app_name=app_name, app_auth=app_auth, source_code=source_code
+    )
